/*****************************************************************************
 *   This example is controlling the LEDs using the joystick
 *
 *   Copyright(C) 2009, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

#include "mcu_regs.h"
#include "lpc_types.h"
#include "board.h"
#include "uart.h"
#include "stdio.h"
#include "timer32.h"
#include "gpio.h"
#include "i2c.h"
#include "ssp.h"
#include "adc.h"

#include "joystick.h"
#include "pca9532.h"
#include "acc.h"
#include "rotary.h"
#include "led7seg.h"
#include "oled.h"
#include "rgb.h"

#include "sysctrl.h"

static uint8_t barPos = 2;

static void moveBar(uint8_t steps, uint8_t dir)
{
    uint16_t ledOn = 0;

    if (barPos == 0)
        ledOn = (1 << 0) | (3 << 14);
    else if (barPos == 1)
        ledOn = (3 << 0) | (1 << 15);
    else
        ledOn = 0x07 << (barPos-2);

    barPos += (dir*steps);
    barPos = (barPos % 16);

    pca9532_setLeds(ledOn, 0xffff);
}

static uint8_t ch7seg = '0';
static void change7Seg(uint8_t rotaryDir)
{

    if (rotaryDir != ROTARY_WAIT) {

        if (rotaryDir == ROTARY_RIGHT) {
            ch7seg++;
        }
        else {
            ch7seg--;
        }

        if (ch7seg > '9')
            ch7seg = '0';
        else if (ch7seg < '0')
            ch7seg = '9';

        led7seg_setChar(ch7seg, FALSE);

    }
}

static void drawOled(uint8_t joyState)
{
    static int wait = 0;
    static uint8_t currX = 48;
    static uint8_t currY = 32;
    static uint8_t lastX = 0;
    static uint8_t lastY = 0;

    if ((joyState & JOYSTICK_CENTER) != 0) {
        oled_clearScreen(OLED_COLOR_BLACK);
        return;
    }

    if (wait++ < 3)
        return;

    wait = 0;

    if ((joyState & JOYSTICK_UP) != 0 && currY > 0) {
        currY--;
    }

    if ((joyState & JOYSTICK_DOWN) != 0 && currY < OLED_DISPLAY_HEIGHT-1) {
        currY++;
    }

    if ((joyState & JOYSTICK_RIGHT) != 0 && currX < OLED_DISPLAY_WIDTH-1) {
        currX++;
    }

    if ((joyState & JOYSTICK_LEFT) != 0 && currX > 0) {
        currX--;
    }

    if (lastX != currX || lastY != currY) {
        oled_putPixel(currX, currY, OLED_COLOR_WHITE);
        lastX = currX;
        lastY = currY;
    }
}

static void changeRgbLeds(uint32_t value)
{
    uint8_t leds = 0;

    leds = value / 128;

    rgb_setLeds(leds);
}

#define SP_PIN_HIGH() (LPC_GPIO1->SET = (0x1<<5))
#define SP_PIN_LOW()  (LPC_GPIO1->CLR = (0x1<<5))


static uint32_t notes[] = {
        2272, // A - 440 Hz
        2024, // B - 494 Hz
        3816, // C - 262 Hz
        3401, // D - 294 Hz
        3030, // E - 330 Hz
        2865, // F - 349 Hz
        2551, // G - 392 Hz
        1136, // a - 880 Hz
        1012, // b - 988 Hz
        1912, // c - 523 Hz
        1703, // d - 587 Hz
        1517, // e - 659 Hz
        1432, // f - 698 Hz
        1275, // g - 784 Hz
};

static void playNote(uint32_t note, uint32_t durationMs) {

    uint32_t t = 0;

    if (note > 0) {

        while (t < (durationMs*1000)) {
            SP_PIN_HIGH();
            brd_delayUs(note / 2);

            SP_PIN_LOW();
            brd_delayUs(note / 2);

            t += note;
        }

    }
    else {
        brd_delayMs(durationMs);
    }
}

static uint32_t getNote(uint8_t ch)
{
    if (ch >= 'A' && ch <= 'G')
        return notes[ch - 'A'];

    if (ch >= 'a' && ch <= 'g')
        return notes[ch - 'a' + 7];

    return 0;
}

static uint32_t getDuration(uint8_t ch)
{
    if (ch < '0' || ch > '9')
        return 400;

    /* number of ms */

    return (ch - '0') * 200;
}

static uint32_t getPause(uint8_t ch)
{
    switch (ch) {
    case '+':
        return 0;
    case ',':
        return 5;
    case '.':
        return 20;
    case '_':
        return 30;
    default:
        return 5;
    }
}

static void playSong(uint8_t *song) {
    uint32_t note = 0;
    uint32_t dur  = 0;
    uint32_t pause = 0;

    /*
     * A song is a collection of tones where each tone is
     * a note, duration and pause, e.g.
     *
     * "E2,F4,"
     */

    while(*song != '\0') {
        note = getNote(*song++);
        if (*song == '\0')
            break;
        dur  = getDuration(*song++);
        if (*song == '\0')
            break;
        pause = getPause(*song++);

        playNote(note, dur);
        brd_delayMs(pause);
    }
}

static uint8_t * song = (uint8_t*)"C2.C2,D4,C4,F4,E8,";
        //(uint8_t*)"C2.C2,D4,C4,F4,E8,C2.C2,D4,C4,G4,F8,C2.C2,c4,A4,F4,E4,D4,A2.A2,H4,F4,G4,F8,";
        //"D4,B4,B4,A4,A4,G4,E4,D4.D2,E4,E4,A4,F4,D8.D4,d4,d4,c4,c4,B4,G4,E4.E2,F4,F4,A4,A4,G8,";

int main (void) {


    int32_t xoff = 0;
    int32_t yoff = 0;
    int32_t zoff = 0;

    int8_t x = 0;

    int8_t y = 0;
    int8_t z = 0;
    uint8_t dir = 1;
    uint8_t wait = 0;

    uint8_t state    = 0;

    uint32_t trim = 0;

    uint8_t btn1 = 0;

    I2C_InitTypeDef i2cInit;
    SSP_InitTypeDef SSP_InitStructure;

    brd_delay_init();
    brd_usb_uart_init();
    brd_trimpot_init();

    UART_SendString(DEV_USB_UART, (uint8_t*)"Demo\r\n");

    i2cInit.ClockRate = 100000;
    i2cInit.Mode      = I2C_MASTER;
    i2cInit.InterruptMode = I2C_INTERRUPT_MODE;
    I2C_Init(&i2cInit);

    SSP_StructInit(&SSP_InitStructure);
    SSP_InitStructure.ClockRate = 1000000;
    SSP_Init(&SSP_InitStructure);
    SSP_Cmd (ENABLE);

    ADC_Init (1000000);
    ADC_ChannelCmd (BRD_ADC_TRIMPOT_CHANNEL, ENABLE);
    ADC_BurstCmd (ENABLE);

    rotary_init();
    led7seg_init();

    pca9532_init();
    joystick_init();
    acc_init();
    oled_init();
    rgb_init();


    /*
     * Assume base board in zero-g position when reading first value.
     */
    acc_read(&x, &y, &z);
    xoff = 0-x;
    yoff = 0-y;
    zoff = 64-z;


    /* ---- Speaker ------> */

    SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO0, ENABLE);
    SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO1, ENABLE);
    SYS_ConfigAHBCLK(SYS_AHBCLKCTRL_GPIO2, ENABLE);

    GPIO_SetDir(LPC_GPIO2, 0, 1);
    GPIO_SetDir(LPC_GPIO2, 4, 1);

    GPIO_SetDir(LPC_GPIO0, 28, 1);
    GPIO_SetDir(LPC_GPIO0, 29, 1);
    GPIO_SetDir(LPC_GPIO1, 6, 1);
    GPIO_SetDir(LPC_GPIO1, 5, 1);

    GPIO_SetLowLevel(LPC_GPIO0, 28, 0); //LM4811-clk
    GPIO_SetLowLevel(LPC_GPIO0, 29, 0); //LM4811-up/dn
    GPIO_SetLowLevel(LPC_GPIO1, 6, 0);  //LM4811-shutdn


    /* <---- Speaker ------ */

    GPIO_SetDir(LPC_GPIO0, 12, 0);

    moveBar(1, dir);
    oled_clearScreen(OLED_COLOR_BLACK);
    while (1) {

        /* ####### Accelerometer and LEDs  ###### */
        /* # */

        acc_read(&x, &y, &z);
        x = x+xoff;
        y = y+yoff;
        z = z+zoff;

        if (y < 0) {
            dir = 1;
            y = -y;
        }
        else {
            dir = -1;
        }

        if (y > 1 && wait++ > (40 / (1 + (y/10)))) {
            moveBar(1, dir);
            wait = 0;
        }

        /* # */
        /* ############################################ */


        /* ####### Rotary and 7-segment display  ###### */
        /* # */

        change7Seg(rotary_read());

        /* # */
        /* ############################################# */


        /* ####### Joystick and OLED  ###### */
        /* # */

        state = joystick_read();
        if (state != 0)
            drawOled(state);

        /* # */
        /* ############################################# */

        btn1 = GPIO_GetPinValue(LPC_GPIO0, 12);

        /* ############ Trimpot and RGB LED  ########### */
        /* # */

        trim = ADC_GetData (BRD_ADC_TRIMPOT_CHANNEL);
        changeRgbLeds(trim);
//#if 0

        /* # */
        /* ############################################# */



        if (btn1 == 0)
            playSong(song);
//#endif
        brd_delayMs(1);
    }


}
